@@ -6,10 +6,10 @@ import base64
 
 '''
 Available styles
-['_classic_test', 'bmh', 'classic', 'dark_background', 'fast', 'fivethirtyeight', 'ggplot', 
-'grayscale', 'seaborn-bright', 'seaborn-colorblind', 'seaborn-dark-palette', 'seaborn-dark', 
-'seaborn-darkgrid', 'seaborn-deep', 'seaborn-muted', 'seaborn-notebook', 'seaborn-paper', 
-'seaborn-pastel', 'seaborn-poster', 'seaborn-talk', 'seaborn-ticks', 'seaborn-white', 
+['_classic_test', 'bmh', 'classic', 'dark_background', 'fast', 'fivethirtyeight', 'ggplot',
+'grayscale', 'seaborn-bright', 'seaborn-colorblind', 'seaborn-dark-palette', 'seaborn-dark',
+'seaborn-darkgrid', 'seaborn-deep', 'seaborn-muted', 'seaborn-notebook', 'seaborn-paper',
+'seaborn-pastel', 'seaborn-poster', 'seaborn-talk', 'seaborn-ticks', 'seaborn-white',
 'seaborn-whitegrid', 'seaborn', 'Solarize_Light2', 'tableau-colorblind10']
 '''
 
@@ -23,23 +23,23 @@ colors = ['sienna',     'gold',
 
 '''
 Colormaps:
-Accent, Accent_r, Blues, Blues_r, 
-BrBG, BrBG_r, BuGn, BuGn_r, BuPu, BuPu_r, CMRmap, CMRmap_r, Dark2, Dark2_r, GnBu, GnBu_r, 
-Greens, Greens_r, Greys, Greys_r, OrRd, OrRd_r, Oranges, Oranges_r, PRGn, PRGn_r, Paired, 
-Paired_r, Pastel1, Pastel1_r, Pastel2, Pastel2_r, PiYG, PiYG_r, PuBu, PuBuGn, PuBuGn_r, 
-PuBu_r, PuOr, PuOr_r, PuRd, PuRd_r, Purples, Purples_r, RdBu, RdBu_r, RdGy, RdGy_r, RdPu, 
-RdPu_r, RdYlBu, RdYlBu_r, RdYlGn, RdYlGn_r, Reds, Reds_r, Set1, Set1_r, Set2, Set2_r, Set3, 
-Set3_r, Spectral, Spectral_r, Wistia, Wistia_r, YlGn, YlGnBu, YlGnBu_r, YlGn_r, YlOrBr, 
+Accent, Accent_r, Blues, Blues_r,
+BrBG, BrBG_r, BuGn, BuGn_r, BuPu, BuPu_r, CMRmap, CMRmap_r, Dark2, Dark2_r, GnBu, GnBu_r,
+Greens, Greens_r, Greys, Greys_r, OrRd, OrRd_r, Oranges, Oranges_r, PRGn, PRGn_r, Paired,
+Paired_r, Pastel1, Pastel1_r, Pastel2, Pastel2_r, PiYG, PiYG_r, PuBu, PuBuGn, PuBuGn_r,
+PuBu_r, PuOr, PuOr_r, PuRd, PuRd_r, Purples, Purples_r, RdBu, RdBu_r, RdGy, RdGy_r, RdPu,
+RdPu_r, RdYlBu, RdYlBu_r, RdYlGn, RdYlGn_r, Reds, Reds_r, Set1, Set1_r, Set2, Set2_r, Set3,
+Set3_r, Spectral, Spectral_r, Wistia, Wistia_r, YlGn, YlGnBu, YlGnBu_r, YlGn_r, YlOrBr,
 YlOrBr_r, YlOrRd, YlOrRd_r, afmhot, afmhot_r, autumn, autumn_r, binary, binary_r, bone,
- bone_r, brg, brg_r, bwr, bwr_r, cividis, cividis_r, cool, cool_r, coolwarm, coolwarm_r, 
- copper, copper_r, cubehelix, cubehelix_r, flag, flag_r, gist_earth, gist_earth_r, gist_gray, 
- gist_gray_r, gist_heat, gist_heat_r, gist_ncar, gist_ncar_r, gist_rainbow, gist_rainbow_r, 
+ bone_r, brg, brg_r, bwr, bwr_r, cividis, cividis_r, cool, cool_r, coolwarm, coolwarm_r,
+ copper, copper_r, cubehelix, cubehelix_r, flag, flag_r, gist_earth, gist_earth_r, gist_gray,
+ gist_gray_r, gist_heat, gist_heat_r, gist_ncar, gist_ncar_r, gist_rainbow, gist_rainbow_r,
  gist_stern, gist_stern_r, gist_yarg, gist_yarg_r, gnuplot, gnuplot2, gnuplot2_r, gnuplot_r,
-  gray, gray_r, hot, hot_r, hsv, hsv_r, icefire, icefire_r, inferno, inferno_r, jet, jet_r, 
-  magma, magma_r, mako, mako_r, nipy_spectral, nipy_spectral_r, ocean, ocean_r, pink, pink_r, 
-  plasma, plasma_r, prism, prism_r, rainbow, rainbow_r, rocket, rocket_r, seismic, seismic_r, 
-  spring, spring_r, summer, summer_r, tab10, tab10_r, tab20, tab20_r, tab20b, tab20b_r, tab20c, 
-  tab20c_r, terrain, terrain_r, twilight, twilight_r, twilight_shifted, twilight_shifted_r, 
+  gray, gray_r, hot, hot_r, hsv, hsv_r, icefire, icefire_r, inferno, inferno_r, jet, jet_r,
+  magma, magma_r, mako, mako_r, nipy_spectral, nipy_spectral_r, ocean, ocean_r, pink, pink_r,
+  plasma, plasma_r, prism, prism_r, rainbow, rainbow_r, rocket, rocket_r, seismic, seismic_r,
+  spring, spring_r, summer, summer_r, tab10, tab10_r, tab20, tab20_r, tab20b, tab20b_r, tab20c,
+  tab20c_r, terrain, terrain_r, twilight, twilight_r, twilight_shifted, twilight_shifted_r,
   viridis, viridis_r, vlag, vlag_r, winter, winter_r
 '''
 
@@ -58,11 +58,11 @@ def to_png_b64(fig, dpi = 150):
     """
     output = io.BytesIO()
     fig.savefig(output, dpi = dpi, bbox_inches = 'tight')
-    
+
     # Encode PNG image to base64 string
     pngImageB64String = "data:image/png;base64,"
     pngImageB64String += base64.b64encode(output.getvalue()).decode('utf8')
-    
+
     return pngImageB64String
 
 def prepare_data(test, traces, options):
@@ -71,21 +71,21 @@ def prepare_data(test, traces, options):
 
     # Dataframe to return
     df = DataFrame()
-        
+
     # Check if there are different subplots
     n_subplots = 1
 
     for trace in traces:
-        if 'subplot' in traces[trace].keys(): 
+        if 'subplot' in traces[trace].keys():
             n_subplots = max(n_subplots, traces[trace]['subplot'])
-        else: 
+        else:
             std_out (f'Trace {trace} not assigned to subplot. Skipping', 'WARNING')
 
     std_out (f'Making {n_subplots} subplots')
-    
+
     # Generate list of subplots
     subplots = [[] for x in range(n_subplots)]
-    
+
     # Put data in the df
     for trace in traces.keys():
 
@@ -109,7 +109,7 @@ def prepare_data(test, traces, options):
             if nchans == 'all': channels = list(test.devices[ndev].readings.columns)
             elif type(nchans) == str: channels = [nchans]
             else: channels = nchans
-            
+
             for channel in channels:
                 # Check if device is in columns
                 if channel not in test.devices[ndev].readings.columns:
@@ -131,7 +131,7 @@ def prepare_data(test, traces, options):
                     else:
                         column_orig.append(col_name)
                         columns_add.append(col_name)
-                
+
                 # Device dataframe
                 dfdev = DataFrame(test.devices[ndev].readings[column_orig].values,
                                 columns = columns_add,
@@ -167,13 +167,13 @@ def prepare_data(test, traces, options):
         # https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.core.resample.Resampler.aggregate.html
         if 'extras' in traces[trace]:
             for extra in traces[trace]['extras']:
-                
+
                 extra_name = channel + f'-{extra.upper()}'
                 sbl = subplots[traces[trace]['subplot']-1]
-                
+
                 if extra == 'max':
                     df[extra_name] = df.loc[:, sbl].max(axis = 1)
-                
+
                 if extra == 'mean':
                     df[extra_name] = df.loc[:, sbl].mean(axis = 1)
 
@@ -191,11 +191,14 @@ def prepare_data(test, traces, options):
     # df = df.apply(to_numeric, errors='coerce')
     df = df.astype(float, errors='ignore')
 
+    if df.empty:
+        std_out('Empty dataframe for plot', 'ERROR')
+        return None, None
     # Resample it
-    if options['frequency'] is not None: 
+    if options['frequency'] is not None:
         std_out(f"Resampling at {options['frequency']}", "INFO")
 
-        if 'resample' in options: 
+        if 'resample' in options:
 
             if options['resample'] == 'max': df = df.resample(options['frequency']).max()
             if options['resample'] == 'min': df = df.resample(options['frequency']).min()
@@ -208,7 +211,7 @@ def prepare_data(test, traces, options):
     if options['clean_na'] is not None:
         if options['clean_na'] == 'fill':
             df = df.fillna(method='ffill')
-        if options['clean_na'] == 'drop':              
+        if options['clean_na'] == 'drop':
             df.dropna(axis = 0, how='any')
 
     if df.empty: std_out('Dataframe for selected options is empty', 'WARNING')
@@ -220,9 +223,9 @@ def groupby_session(dataframe, **kwargs):
     Prepares datafram with groupby options for plots in heatmap or boxplots.
     Parameters
     ----------
-        dataframe: pd.DataFrame() 
+        dataframe: pd.DataFrame()
             Containing the channel and index to groupby.
-        frequency_hours: int 
+        frequency_hours: int
             6
             Bin length in hours
         # TODO - do generic for not only hours
@@ -274,17 +277,17 @@ def groupby_session(dataframe, **kwargs):
 
     channel = df.columns[0]
     df = df.assign(session = cut(df.index.hour, arange(0, 25, freq_time), labels = labels, right = False))
-    
+
     # Group them by session
     df_session = df.groupby(['session']).mean()
     df_session = df_session[channel]
 
     ## Full dataframe
     list_all = ['session', channel]
-    
+
     if periods is not None: list_all.append('period')
 
-    # TODO    
+    # TODO
     # # Check relative measurements
     # if self.options['relative']:
     #     # Calculate average
@@ -296,9 +299,9 @@ def groupby_session(dataframe, **kwargs):
     #         if column != 'session':
     #             df[column + '_REL'] = df[column]/df_session_avg
     #             list_all.append(column + '_REL')
-        
+
     ## Full dataframe
     df = df[list_all]
     df.dropna(axis = 0, how='all', inplace = True)
 
-    return df, labels, label_cat, channel
\ No newline at end of file
+    return df, labels, label_cat, channel
