@@ -21,15 +21,24 @@ def enablePrint():
 
 if '-h' in sys.argv or '--help' in sys.argv or '-help' in sys.argv:
     print('USAGE:\n\nresgister.py [options] action[s]')
-    print('\noptions: -v: verbose')
+    print('\noptions: -v: verbose; -t: test device')
     print('actions: register, inventory')
     print('register options: -n platform_name -i kit_blueprint_id (default: 26)')
     print('inventory -d "description" --with-test [y/n] (default: n)')
+    print('-p port [-f]: specify a port instead of scanning')
+    print('-f: option ignores serial device description (must contain Smartcitizen otherwise)')
     sys.exit()
 
 import sck
 kit = sck.sck(to_register = True)
-kit.begin() 
+
+force = False
+port = None
+if '-p' in sys.argv:
+    port = sys.argv[sys.argv.index('-p')+1]
+    if '-f' in sys.argv: force = True
+elif '-f' in sys.argv: ERROR('No force action if port is not specified'); sys.exit()
+if not kit.begin(port=port, force=force): sys.exit()
 
 verbose = False
 blockPrint()
@@ -54,6 +63,10 @@ if 'register' in sys.argv:
             sys.exit()
         kit.blueprint_id = bid
 
+    if '-t' in sys.argv:
+        print ('Setting test device')
+        kit.is_test = True
+
     kit.platform_name = kit.platform_name + ' #' + kit.esp_macAddress[-5:].replace(':', '')
     kit.register()
 
@@ -62,7 +75,7 @@ if 'register' in sys.argv:
     print("Mac address: " + kit.esp_macAddress)
     print("Device token: " + kit.token)
     print("Platform kit name: " + kit.platform_name)
-    print("Platform page:" + kit.platform_url)
+    print("Platform page: " + kit.platform_url)
 
 if 'inventory' in sys.argv:
     try:
@@ -86,39 +99,38 @@ if 'inventory' in sys.argv:
     if not hasattr(kit, 'platform_url'):
         kit.platform_url = ''
 
-    local_inv_path = os.path.join(inventory_path, 'deliveries')
     s3_inv_path = "inventory/deliveries"
     local_inv_name = "inventory.csv"
-    if not os.path.exists(local_inv_path): os.makedirs(local_inv_path)
+    if not os.path.exists(inventory_path): os.makedirs(inventory_path)
 
     try:
         # Try to download file from S3
         sync = S3handler()
-        sync.download(os.path.join(local_inv_path, local_inv_name), os.path.join(s3_inv_path, local_inv_name))
+        sync.download(os.path.join(inventory_path, local_inv_name), os.path.join(s3_inv_path, local_inv_name))
     except:
         # Keep things local
         print_exc()
         print('Problem downloading file from S3, using local file')
 
-        if os.path.exists(os.path.join(local_inv_path, local_inv_name)):
-            shutil.copyfile(os.path.join(local_inv_path, local_inv_name), local_inv_path+".BAK")
-            csvFile = open(os.path.join(local_inv_path, local_inv_name), "a")
+        if os.path.exists(os.path.join(inventory_path, local_inv_name)):
+            shutil.copyfile(os.path.join(inventory_path, local_inv_name), inventory_path+".BAK")
+            csvFile = open(os.path.join(inventory_path, local_inv_name), "a")
         else:
-            csvFile = open(os.path.join(local_inv_path, local_inv_name), "w")
+            csvFile = open(os.path.join(inventory_path, local_inv_name), "w")
             csvFile.write("time,serial,mac,sam_firmVer,esp_firmVer,description,token,platform_name,platform_url,tested,validated,min_validation_date,max_validation_date,replacement,test,destination,batch\n")
         pass
     else:
         # Open the file 
         print ('File from S3 synced correctly')
-        csvFile = open(os.path.join(local_inv_path, local_inv_name), "a")
+        csvFile = open(os.path.join(inventory_path, local_inv_name), "a")
 
     csvFile.write(time.strftime("%Y-%m-%dT%H:%M:%SZ,", time.gmtime()))
     csvFile.write(kit.sam_serialNum + ',' + kit.esp_macAddress + ',' + kit.sam_firmVer + ',' + kit.esp_firmVer + ',' + kit.description + ',' + kit.token + ',' + kit.platform_name + ',' + kit.platform_url + ',' + tested + ',' + ',' + ',' +',' + ',' +',' + ',' +'\n')
     csvFile.close()
 
     # Put the file in S3
-    sync = S3handler()
-    resp = sync.upload(os.path.join(local_inv_path, local_inv_name), os.path.join(s3_inv_path, local_inv_name))
+    if sync is not None:
+        resp = sync.upload(os.path.join(inventory_path, local_inv_name), os.path.join(s3_inv_path, local_inv_name))
 
-    if resp is None: print ('No response, review bucket')
-    else: print ('Success!')
\ No newline at end of file
+        if resp is None: print ('No response, review bucket')
+        else: print ('Success!')
\ No newline at end of file
